var Path = require("path"),
	fs = require('fs'),
	through = require("through2"),
	gutil = require("gulp-util"),
	VersionTag = require('./util');


const PLUGIN_NAME = 'gulp-version-tag';

module.exports = function (dirname, packageRelativePath, options) {
	"use strict";
	var flag = 0,
		versionTag;

	options = options || {};
	options.beforeText = options.beforeText || '-v';
	options.afterText = options.afterText || '';
	options.reuse = options.reuse || false;

	var errorHandle = function (err) {
		throw new gutil.PluginError(PLUGIN_NAME, err);
	};
	versionTag = new VersionTag(dirname, packageRelativePath, errorHandle);


	if (!packageRelativePath) {
		throw new gutil.PluginError(PLUGIN_NAME, "Package.json path is empty");
	}

	if (versionTag.version == null) {
		throw new gutil.PluginError(PLUGIN_NAME, "Version is null");
	}


	function gulpVersionTag(file, enc, callback) {
		if (file.isNull()) {
			this.push(file);
			versionTag.save();
			return callback();
		}

		if (file.isBuffer()) {

			setVersionTag(file);
			this.push(file);

		}

		if (file.isStream()) {
			setVersionTag(file);
			this.push(file);
		}

		function setVersionTag(file) {
			var extname = Path.extname(file.path);
			var basename = Path.basename(file.path, extname);
			var dirname = Path.dirname(file.path);
			if (options.reuse && global.versionTag) {
				basename += options.beforeText + global.versionTag + options.afterText;
				file.path = Path.join(dirname, basename + extname);
			}
			else {
				if (flag == 0) {
					versionTag.patch();
					flag++;
				}
				global.versionTag = versionTag.version;
				basename += options.beforeText + versionTag.version + options.afterText;
				file.path = Path.join(dirname, basename + extname);
			}
		}

		return callback();
	}

	return through.obj(gulpVersionTag);
};
